@@ -24,6 +24,13 @@ estimate_model <- function(df, dl) {
   se <- NULL
   p <- NULL
   omit_vars <- NULL
+
+  if (length(feffects) > 2)
+    stop("Only up to two-dimensional fixed effect structures are supported")
+
+  if (clusters[1] != "" & any(! clusters %in% feffects))
+    stop("Only fixed effect variables can be used as clusters.")
+
   if (type_str == "auto") {
     if(!(is.factor(df[,dv]) | is.logical(df[,dv]))) type_str <- "ols"
     else type_str <- "logit"
@@ -32,18 +39,7 @@ estimate_model <- function(df, dl) {
     df[,feffects] <- lapply(as.data.frame(df[,feffects]), function (x) factor(x, ordered = FALSE))
   }
   if(type_str == "ols") {
-    if (feffects[1] != "" & clusters[1] != "") {
-      f <- stats::as.formula(paste(dv, "~", paste(idvs, collapse = " + "), " | ",
-                                   paste(feffects, collapse = " + "), " | 0 | ", paste(clusters, collapse = " + ")))
-    } else if (feffects[1] != "") {
-      f <- stats::as.formula(paste(dv, "~", paste(idvs, collapse = " + "), " | ",
-                                   paste(feffects, collapse = " + ")))
-    } else if (clusters[1] != "") {
-      f <- stats::as.formula(paste(dv, "~", paste(idvs, collapse = " + "), " | 0 | 0 | ",
-                                   paste(clusters, collapse = " + ")))
-    } else {
-      f <- stats::as.formula(paste(dv, "~", paste(idvs, collapse = " + ")))
-    }
+    f <- stats::as.formula(paste(dv, "~", paste(idvs, collapse = " + ")))
   } else if (nlevels(as.factor(df[,dv])) > 2) {
     stop("multinomial logit is not implemented. Sorry.")
   } else {
@@ -65,12 +61,52 @@ estimate_model <- function(df, dl) {
       p <- test[,4]
     }
     if (feffects[1] != "") {
-        omit_vars <- (length(idvs) + 1):length(model$coefficients)
+        omit_vars <- names(model$coefficients)[! names(model$coefficients) %in% idvs]
     }
   } else {
-    model <- lfe::felm(f, data=df, psdef=FALSE)
+    if (feffects[1] == "") {
+      model <- stats::lm(f, data=df)
+    } else {
+      # plm() is much faster when the dimension with more units is the first
+      # index member. Why I have no idea.
+      if (length(feffects) > 1) {
+        if (
+          length(unique(df[, feffects[2]])) > length(unique(df[ ,feffects[1]]))
+        ) {
+          feffects <- c(feffects[2], feffects[1])
+        }
+      }
+      df <- plm::pdata.frame(df, feffects)
+      if (length(feffects) == 1) {
+        model <- plm::plm(f, df, model = "within", effect = "individual")
+        if (clusters[1] != "") {
+          vcov <- plm::vcovHC(model, type="sss", cluster="group")
+          test <- lmtest::coeftest(model, vcov)
+          se <- test[,2]
+          p <- test[,4]
+        }
+      } else {
+        model <- plm::plm(f, df, model = "within", effect = "twoways")
+        if (clusters[1] != "") {
+          if (length(clusters) == 2) {
+            vcov <- plm::vcovDC(model, type="sss")
+          } else if (clusters[1] == feffects[1]) {
+            vcov <- plm::vcovHC(model, type="sss", cluster="group")
+          } else vcov <- plm::vcovHC(model, type="sss", cluster="time")
+          test <- lmtest::coeftest(model, vcov)
+          se <- test[,2]
+          p <- test[,4]
+        }
+      }
+      # Stargazer identifies plm models based on call (wout package) ... sigh
+      model$call[1] <- parse(text = "plm()")[[1]]
+    }
   }
-  list(model = model, type_str = type_str, fe_str = fe_str, cl_str = cl_str, p = p, se = se, omit_vars = omit_vars)
+
+  list(
+    model = model, type_str = type_str, fe_str = fe_str, cl_str = cl_str,
+    p = p, se = se, omit_vars = omit_vars
+  )
 }
 
 
@@ -96,15 +132,20 @@ estimate_model <- function(df, dl) {
 #' }
 #'
 #' @details
-#' This is a wrapper function calling the stargazer package. Depending on whether the dependent variable
-#'   is numeric, logical or a factor with two levels, the models are estimated
-#'   using \code{\link[lfe]{felm}} (for numeric dependent variables)
-#'   or \code{\link[stats]{glm}} (with \code{family = binomial(link="logit")}) (for two-level factors or logical variables).
-#'   You can override this behavior by specifying the model with the \code{models} parameter.
-#'   Multinomial logit models are not supported.
-#'   For \code{\link[stats]{glm}}, clustered standard errors are estimated using
-#'   \code{\link[multiwayvcov]{cluster.vcov}}.
-#'   If run with \code{byvar}, only levels that have more observations than coefficients are estimated.
+#' This is a wrapper function calling the stargazer package. For numeric
+#'   dependent variables the models are estimated using \code{\link[stats]{lm}}
+#'   for models without and \code{\link[plm]{plm}} for models with fixed
+#'   effects. Binary dependent variable models are estimated using
+#'   \code{\link[stats]{glm}} (with \code{family = binomial(link="logit")}).
+#'   You can override this behavior by specifying the model with the
+#'   \code{models} parameter. Multinomial logit models are not supported.
+#'   Clustered standard errors are estimated using \code{plm}'s robust
+#'   covariance matrix estimators for OLS and
+#'   \code{\link[multiwayvcov]{cluster.vcov}} for logit models.
+#'   Only up to two dimensions are supported for fixed effects and standard
+#'   error clusters need to be also present as fixed effects.
+#'   If run with \code{byvar}, only levels that have more observations than
+#'   coefficients are estimated.
 #'
 #'
 #' @examples
@@ -134,6 +175,7 @@ prepare_regression_table <- function(df, dvs, idvs, feffects = rep("", length(dv
   if ((length(dvs) > 1) &&
       ((length(dvs) != length(idvs)) | (length(dvs) != length(feffects)) | (length(dvs) != length(clusters) | (length(dvs) != length(models)))))
     stop("'dvs', 'idvs', 'feffects', 'clusters' and 'models' need to be of equal lenghth.")
+
   datalist <- list()
   if (byvar != "") {
     datalist <- list(dvs = dvs,
@@ -204,12 +246,18 @@ prepare_regression_table <- function(df, dvs, idvs, feffects = rep("", length(dv
     if (models[[i]]$cl_str != "")  cl_str <- c(cl_str, models[[i]]$cl_str)
     else cl_str <- c(cl_str, "No")
 
-    n_str <- c(n_str, format(m[[i]]$N, big.mark = ","))
+    n_str <- c(n_str, format(stats::nobs(m[[i]]), big.mark = ","))
 
     if (models[[i]]$type_str == "ols") {
       ols_present <- TRUE
-      r2_str <- c(r2_str, sprintf("%.3f", summary(m[[i]])$r.squared))
-      adjr2_str <- c(adjr2_str, sprintf("%.3f", summary(m[[i]])$adj.r.squared))
+      if (models[[i]]$fe_str == "") {
+        r2_str <- c(r2_str, sprintf("%.3f", summary(m[[i]])$r.squared))
+        adjr2_str <- c(adjr2_str, sprintf("%.3f", summary(m[[i]])$adj.r.squared))
+      } else {
+        r2_str <- c(r2_str, sprintf("%.3f", summary(m[[i]])$r.squared)[1])
+        adjr2_str <- c(adjr2_str, sprintf("%.3f", summary(m[[i]])$r.squared)[2])
+      }
+
       success_str <- c(success_str, "")
       pr2_str <- c(pr2_str, "")
     }
@@ -248,6 +296,9 @@ prepare_regression_table <- function(df, dvs, idvs, feffects = rep("", length(dv
     add.lines <- list(mt_str, success_str, fe_str, cl_str, pr2_str)
   else add.lines <- list(mt_str, fe_str, cl_str, n_str, r2_str, adjr2_str)
 
+  if (!is.null(omit_vars)) {
+    omit_vars <- unique(c("^Constant$", paste0("^", omit_vars, "$")))
+  }
   if (byvar != "") {
     # Stargazer 5.2.2 seems to have a bug on how column.labels are treated in text/html
     # Escaping labels does not help and even sometimes introduces an error.
